@@ -6,10 +6,10 @@
  * between the v1 and v2 implementations.  One of the things it establishes is
  * what protocol is being used.
  *
- * Written by Russ Allbery <rra@stanford.edu>
+ * Written by Russ Allbery <eagle@eyrie.org>
  * Based on work by Anton Ushakov
- * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012, 2013
- *     The Board of Trustees of the Leland Stanford Junior University
+ * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012, 2013,
+ *     2014 The Board of Trustees of the Leland Stanford Junior University
  *
  * See LICENSE for licensing terms.
  */
@@ -113,8 +113,7 @@ internal_import_name(struct remctl *r, const char *host,
     else
         oid = GSS_C_NT_HOSTBASED_SERVICE;
     major = gss_import_name(&minor, &name_buffer, oid, name);
-    if (defprinc != NULL)
-        free(defprinc);
+    free(defprinc);
     if (major != GSS_S_COMPLETE) {
         internal_gssapi_error(r, "parsing name", major, minor);
         return false;
@@ -146,6 +145,8 @@ internal_set_cred(struct remctl *r, gss_cred_id_t *gss_cred)
             return false;
         }
     }
+    if (r->krb_ccache != NULL)
+        krb5_cc_close(r->krb_ctx, r->krb_ccache);
     code = krb5_cc_resolve(r->krb_ctx, r->ccache, &r->krb_ccache);
     if (code != 0) {
         internal_krb5_error(r, "opening ticket cache", code);
@@ -290,6 +291,8 @@ internal_open(struct remctl *r, const char *host, const char *principal)
     r->context = gss_context;
     r->ready = 0;
     gss_release_name(&minor, &name);
+    if (gss_cred != GSS_C_NO_CREDENTIAL)
+        gss_release_cred(&minor, &gss_cred);
     return true;
 
 fail:
@@ -297,6 +300,8 @@ fail:
     r->fd = INVALID_SOCKET;
     if (name != GSS_C_NO_NAME)
         gss_release_name(&minor, &name);
+    if (gss_cred != GSS_C_NO_CREDENTIAL)
+        gss_release_cred(&minor, &gss_cred);
     if (gss_context != GSS_C_NO_CONTEXT)
         gss_delete_sec_context(&minor, &gss_context, GSS_C_NO_BUFFER);
     return false;
